@@ -3,14 +3,18 @@
  *
  * All rights reserved.
  *
- * This file is part of ObjFW. It may be distributed under the terms of the
- * Q Public License 1.0, which can be found in the file LICENSE.QPL included in
- * the packaging of this file.
+ * This program is free software: you can redistribute it and/or modify it
+ * under the terms of the GNU Lesser General Public License version 3.0 only,
+ * as published by the Free Software Foundation.
  *
- * Alternatively, it may be distributed under the terms of the GNU General
- * Public License, either version 2 or 3, which can be found in the file
- * LICENSE.GPLv2 or LICENSE.GPLv3 respectively included in the packaging of this
- * file.
+ * This program is distributed in the hope that it will be useful, but WITHOUT
+ * ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or
+ * FITNESS FOR A PARTICULAR PURPOSE. See the GNU Lesser General Public License
+ * version 3.0 for more details.
+ *
+ * You should have received a copy of the GNU Lesser General Public License
+ * version 3.0 along with this program. If not, see
+ * <https://www.gnu.org/licenses/>.
  */
 
 #include "config.h"
@@ -25,6 +29,7 @@
 #import "OFStream.h"
 #import "OFTarArchive.h"
 #import "OFZIPArchive.h"
+#import "OFZooArchive.h"
 
 #import "OFInvalidArgumentException.h"
 #import "OFOpenItemFailedException.h"
@@ -80,7 +85,9 @@ initPathAllowedCharacters(void)
 	}
 
 	percentEncodedPath = IRI.percentEncodedPath;
-	pos = [percentEncodedPath rangeOfString: @"!"].location;
+	pos = [percentEncodedPath
+	    rangeOfString: @"!"
+		  options: OFStringSearchBackwards].location;
 
 	if (pos == OFNotFound)
 		@throw [OFInvalidArgumentException exception];
@@ -127,6 +134,21 @@ initPathAllowedCharacters(void)
 								mode: mode];
 
 		stream = [archive streamForReadingFile: path];
+	} else if ([scheme isEqual: @"zoo"]) {
+		OFZooArchive *archive = [OFZooArchive archiveWithIRI: archiveIRI
+								mode: mode];
+		OFZooArchiveEntry *entry;
+
+		while ((entry = [archive nextEntry]) != nil) {
+			if ([entry.fileName isEqual: path]) {
+				stream = [archive streamForReadingCurrentEntry];
+				goto end;
+			}
+		}
+
+		@throw [OFOpenItemFailedException exceptionWithIRI: IRI
+							      mode: mode
+							     errNo: ENOENT];
 	} else
 		@throw [OFInvalidArgumentException exception];
 
@@ -179,19 +201,15 @@ OFArchiveIRIHandlerIRIForFileInArchive(OFString *scheme,
 	static OFOnceControl onceControl = OFOnceControlInitValue;
 	OFMutableIRI *ret = [OFMutableIRI IRIWithScheme: scheme];
 	void *pool = objc_autoreleasePoolPush();
-	OFString *archiveIRIString;
 
 	OFOnce(&onceControl, initPathAllowedCharacters);
 
 	pathInArchive = [pathInArchive
 	    stringByAddingPercentEncodingWithAllowedCharacters:
 	    pathAllowedCharacters];
-	archiveIRIString = [archiveIRI.string
-	    stringByAddingPercentEncodingWithAllowedCharacters:
-	    pathAllowedCharacters];
 
 	ret.percentEncodedPath = [OFString
-	    stringWithFormat: @"%@!%@", archiveIRIString, pathInArchive];
+	    stringWithFormat: @"%@!%@", archiveIRI.string, pathInArchive];
 	[ret makeImmutable];
 
 	objc_autoreleasePoolPop(pool);
