@@ -3,14 +3,18 @@
  *
  * All rights reserved.
  *
- * This file is part of ObjFW. It may be distributed under the terms of the
- * Q Public License 1.0, which can be found in the file LICENSE.QPL included in
- * the packaging of this file.
+ * This program is free software: you can redistribute it and/or modify it
+ * under the terms of the GNU Lesser General Public License version 3.0 only,
+ * as published by the Free Software Foundation.
  *
- * Alternatively, it may be distributed under the terms of the GNU General
- * Public License, either version 2 or 3, which can be found in the file
- * LICENSE.GPLv2 or LICENSE.GPLv3 respectively included in the packaging of this
- * file.
+ * This program is distributed in the hope that it will be useful, but WITHOUT
+ * ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or
+ * FITNESS FOR A PARTICULAR PURPOSE. See the GNU Lesser General Public License
+ * version 3.0 for more details.
+ *
+ * You should have received a copy of the GNU Lesser General Public License
+ * version 3.0 along with this program. If not, see
+ * <https://www.gnu.org/licenses/>.
  */
 
 #define OF_LHA_ARCHIVE_M
@@ -72,7 +76,7 @@ OF_DIRECT_MEMBERS
 	OFStringEncoding _encoding;
 	OFSeekableStream *_stream;
 	OFStreamOffset _headerOffset;
-	uint32_t _bytesWritten;
+	uint64_t _bytesWritten;
 	uint16_t _CRC16;
 }
 
@@ -134,8 +138,6 @@ OF_DIRECT_MEMBERS
 			 */
 			[(OFSeekableStream *)_stream seekToOffset: -1
 							   whence: OFSeekEnd];
-
-		_encoding = OFStringEncodingISO8859_1;
 	} @catch (id e) {
 		[self release];
 		@throw e;
@@ -298,7 +300,7 @@ OF_DIRECT_MEMBERS
 	@try {
 		[_lastReturnedStream close];
 	} @catch (OFNotOpenException *e) {
-		/* Might have already been closed by the user - that's fine */
+		/* Might have already been closed by the user - that's fine. */
 	}
 
 	/* LHA archives should be terminated with a header of size 0 */
@@ -343,8 +345,15 @@ OF_DIRECT_MEMBERS
 			    of_initWithStream: stream
 				 distanceBits: 5
 			       dictionaryBits: 17];
+		else if ([compressionMethod isEqual: @"-lhx-"])
+			_decompressedStream = [[OFLHADecompressingStream alloc]
+			    of_initWithStream: stream
+				 distanceBits: 5
+			       dictionaryBits: 20];
 		else if ([compressionMethod isEqual: @"-lh0-"] ||
-		    [compressionMethod isEqual: @"-lhd-"])
+		    [compressionMethod isEqual: @"-lhd-"] ||
+		    [compressionMethod isEqual: @"-lz4-"] ||
+		    [compressionMethod isEqual: @"-pm0-"])
 			_decompressedStream = [stream retain];
 		else
 			@throw [OFUnsupportedVersionException
@@ -409,9 +418,9 @@ OF_DIRECT_MEMBERS
 
 		if (_CRC16 != _entry.CRC16) {
 			OFString *actualChecksum = [OFString stringWithFormat:
-			    @"%04" PRIX16, _CRC16];
+			    @"%04" @PRIX16, _CRC16];
 			OFString *expectedChecksum = [OFString stringWithFormat:
-			    @"%04" PRIX16, _entry.CRC16];
+			    @"%04" @PRIX16, _entry.CRC16];
 
 			@throw [OFChecksumMismatchException
 			    exceptionWithActualChecksum: actualChecksum
@@ -548,7 +557,7 @@ OF_DIRECT_MEMBERS
 	if (_stream == nil)
 		@throw [OFNotOpenException exceptionWithObject: self];
 
-	if (UINT32_MAX - _bytesWritten < length)
+	if (UINT64_MAX - _bytesWritten < length)
 		@throw [OFOutOfRangeException exception];
 
 	@try {
@@ -556,7 +565,7 @@ OF_DIRECT_MEMBERS
 	} @catch (OFWriteFailedException *e) {
 		OFEnsure(e.bytesWritten <= length);
 
-		_bytesWritten += (uint32_t)e.bytesWritten;
+		_bytesWritten += (uint64_t)e.bytesWritten;
 		_CRC16 = OFCRC16(_CRC16, buffer, e.bytesWritten);
 
 		if (e.errNo == EWOULDBLOCK || e.errNo == EAGAIN)
@@ -565,7 +574,7 @@ OF_DIRECT_MEMBERS
 		@throw e;
 	}
 
-	_bytesWritten += (uint32_t)length;
+	_bytesWritten += (uint64_t)length;
 	_CRC16 = OFCRC16(_CRC16, buffer, length);
 
 	return length;
