@@ -3,14 +3,18 @@
  *
  * All rights reserved.
  *
- * This file is part of ObjFW. It may be distributed under the terms of the
- * Q Public License 1.0, which can be found in the file LICENSE.QPL included in
- * the packaging of this file.
+ * This program is free software: you can redistribute it and/or modify it
+ * under the terms of the GNU Lesser General Public License version 3.0 only,
+ * as published by the Free Software Foundation.
  *
- * Alternatively, it may be distributed under the terms of the GNU General
- * Public License, either version 2 or 3, which can be found in the file
- * LICENSE.GPLv2 or LICENSE.GPLv3 respectively included in the packaging of this
- * file.
+ * This program is distributed in the hope that it will be useful, but WITHOUT
+ * ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or
+ * FITNESS FOR A PARTICULAR PURPOSE. See the GNU Lesser General Public License
+ * version 3.0 for more details.
+ *
+ * You should have received a copy of the GNU Lesser General Public License
+ * version 3.0 along with this program. If not, see
+ * <https://www.gnu.org/licenses/>.
  */
 
 #include "config.h"
@@ -81,14 +85,17 @@ extern struct Stret OFForward_stret(id, SEL, ...);
 #endif
 
 struct PreIvars {
+#ifdef OF_MSDOS
+	ptrdiff_t offset;
+#endif
 	int retainCount;
 #if !defined(OF_HAVE_ATOMIC_OPS) && !defined(OF_AMIGAOS)
 	OFSpinlock retainCountSpinlock;
 #endif
 };
 
-#define PRE_IVARS_ALIGN ((sizeof(struct PreIvars) + \
-    (OF_BIGGEST_ALIGNMENT - 1)) & ~(OF_BIGGEST_ALIGNMENT - 1))
+#define PRE_IVARS_ALIGN \
+	OFRoundUpToPowerOf2(sizeof(struct PreIvars), OF_BIGGEST_ALIGNMENT)
 #define PRE_IVARS ((struct PreIvars *)(void *)((char *)self - PRE_IVARS_ALIGN))
 
 static struct {
@@ -158,6 +165,33 @@ OFFreeMemory(void *pointer)
 	free(pointer);
 }
 
+#ifdef OF_MSDOS
+/* Unfortunately, DJGPP's memalign() is broken. */
+
+static void *
+alignedAlloc(size_t size, size_t alignment, ptrdiff_t *offset)
+{
+	char *ptr, *aligned;
+
+	if ((ptr = malloc(size + alignment)) == NULL)
+		return NULL;
+
+	aligned = (char *)OFRoundUpToPowerOf2(alignment, (uintptr_t)ptr);
+	*offset = aligned - ptr;
+
+	return aligned;
+}
+
+static void
+alignedFree(void *ptr, ptrdiff_t offset)
+{
+	if (ptr == NULL)
+		return;
+
+	free((void *)((uintptr_t)ptr - offset));
+}
+#endif
+
 #if !defined(HAVE_ARC4RANDOM) && !defined(HAVE_GETRANDOM)
 static void
 initRandom(void)
@@ -332,6 +366,9 @@ OFAllocObject(Class class, size_t extraSize, size_t extraAlignment,
 {
 	OFObject *instance;
 	size_t instanceSize;
+#ifdef OF_MSDOS
+	ptrdiff_t offset;
+#endif
 
 	instanceSize = class_getInstanceSize(class);
 
@@ -340,8 +377,20 @@ OFAllocObject(Class class, size_t extraSize, size_t extraAlignment,
 		    PRE_IVARS_ALIGN + instanceSize) -
 		    PRE_IVARS_ALIGN - instanceSize;
 
-	instance = calloc(1, PRE_IVARS_ALIGN + instanceSize +
+#if defined(OF_WINDOWS)
+	instance = __mingw_aligned_malloc(PRE_IVARS_ALIGN + instanceSize +
+	    extraAlignment + extraSize, OF_BIGGEST_ALIGNMENT);
+#elif defined(OF_MSDOS)
+	instance = alignedAlloc(PRE_IVARS_ALIGN + instanceSize +
+	    extraAlignment + extraSize, OF_BIGGEST_ALIGNMENT, &offset);
+#elif defined(OF_SOLARIS)
+	if (posix_memalign((void **)&instance, OF_BIGGEST_ALIGNMENT,
+	    PRE_IVARS_ALIGN + instanceSize + extraAlignment + extraSize) != 0)
+		instance = NULL;
+#else
+	instance = malloc(PRE_IVARS_ALIGN + instanceSize +
 	    extraAlignment + extraSize);
+#endif
 
 	if OF_UNLIKELY (instance == nil) {
 		object_setClass((id)&allocFailedException,
@@ -349,12 +398,21 @@ OFAllocObject(Class class, size_t extraSize, size_t extraAlignment,
 		@throw (id)&allocFailedException;
 	}
 
+#ifdef OF_MSDOS
+	((struct PreIvars *)instance)->offset = offset;
+#endif
 	((struct PreIvars *)instance)->retainCount = 1;
 
 #if !defined(OF_HAVE_ATOMIC_OPS) && !defined(OF_AMIGAOS)
 	if OF_UNLIKELY (OFSpinlockNew(
 	    &((struct PreIvars *)instance)->retainCountSpinlock) != 0) {
+# if defined(OF_WINDOWS)
+		__mingw_aligned_free(instance);
+# elif defined(OF_MSDOS)
+		alignedFree(instance, offset);
+# else
 		free(instance);
+# endif
 		@throw [OFInitializationFailedException
 		    exceptionWithClass: class];
 	}
@@ -362,12 +420,20 @@ OFAllocObject(Class class, size_t extraSize, size_t extraAlignment,
 
 	instance = (OFObject *)(void *)((char *)instance + PRE_IVARS_ALIGN);
 
+	memset(instance, 0, instanceSize + extraAlignment + extraSize);
+
 	if (!objc_constructInstance(class, instance)) {
 #if !defined(OF_HAVE_ATOMIC_OPS) && !defined(OF_AMIGAOS)
 		OFSpinlockFree(&((struct PreIvars *)(void *)
 		    ((char *)instance - PRE_IVARS_ALIGN))->retainCountSpinlock);
 #endif
+#if defined(OF_WINDOWS)
+		__mingw_aligned_free((char *)instance - PRE_IVARS_ALIGN);
+#elif defined(OF_MSDOS)
+		alignedFree((char *)instance - PRE_IVARS_ALIGN, offset);
+#else
 		free((char *)instance - PRE_IVARS_ALIGN);
+#endif
 		@throw [OFInitializationFailedException
 		    exceptionWithClass: class];
 	}
@@ -1234,7 +1300,13 @@ _references_to_categories_of_OFObject(void)
 	OFSpinlockFree(&PRE_IVARS->retainCountSpinlock);
 #endif
 
+#if defined(OF_WINDOWS)
+	__mingw_aligned_free((char *)self - PRE_IVARS_ALIGN);
+#elif defined(OF_MSDOS)
+	alignedFree((char *)self - PRE_IVARS_ALIGN, PRE_IVARS->offset);
+#else
 	free((char *)self - PRE_IVARS_ALIGN);
+#endif
 }
 
 /* Required to use properties with the Apple runtime */
