@@ -3,14 +3,18 @@
  *
  * All rights reserved.
  *
- * This file is part of ObjFW. It may be distributed under the terms of the
- * Q Public License 1.0, which can be found in the file LICENSE.QPL included in
- * the packaging of this file.
+ * This program is free software: you can redistribute it and/or modify it
+ * under the terms of the GNU Lesser General Public License version 3.0 only,
+ * as published by the Free Software Foundation.
  *
- * Alternatively, it may be distributed under the terms of the GNU General
- * Public License, either version 2 or 3, which can be found in the file
- * LICENSE.GPLv2 or LICENSE.GPLv3 respectively included in the packaging of this
- * file.
+ * This program is distributed in the hope that it will be useful, but WITHOUT
+ * ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or
+ * FITNESS FOR A PARTICULAR PURPOSE. See the GNU Lesser General Public License
+ * version 3.0 for more details.
+ *
+ * You should have received a copy of the GNU Lesser General Public License
+ * version 3.0 along with this program. If not, see
+ * <https://www.gnu.org/licenses/>.
  */
 
 #include "config.h"
@@ -990,6 +994,35 @@ OFSocketAddressString(const OFSocketAddress *address)
 	}
 }
 
+OFString *
+OFSocketAddressDescription(const OFSocketAddress *address)
+{
+	switch (address->family) {
+	case OFSocketAddressFamilyIPv4:
+		return [OFString
+		    stringWithFormat: @"%@:%" PRIu16,
+				      IPv4String(address),
+				      OFSocketAddressIPPort(address)];
+	case OFSocketAddressFamilyIPv6:
+		return [OFString
+		    stringWithFormat: @"[%@]:%" PRIu16,
+				      IPv6String(address),
+				      OFSocketAddressIPPort(address)];
+	case OFSocketAddressFamilyIPX:
+		return [OFString
+		    stringWithFormat: @"%@.%" PRIX16,
+				      IPXString(address),
+				      OFSocketAddressIPXPort(address)];
+	case OFSocketAddressFamilyAppleTalk:
+		return [OFString
+		    stringWithFormat: @"%@." PRIu8,
+				      appleTalkString(address),
+				      OFSocketAddressAppleTalkPort(address)];
+	default:
+		return OFSocketAddressString(address);
+	}
+}
+
 void
 OFSocketAddressSetIPPort(OFSocketAddress *address, uint16_t port)
 {
